@@ -18,6 +18,11 @@ class ColType:
     pass
 
 
+class IKey(ABC):
+    "Interface for ColType, for using a column as a key in data-diff"
+    python_type: type
+
+
 @dataclass
 class PrecisionType(ColType):
     precision: int
@@ -54,7 +59,12 @@ class Float(FractionalType):
     pass
 
 
-class Decimal(FractionalType):
+class IKey(ABC):
+    "Interface for ColType, for using a column as a key in data-diff"
+    python_type: type
+
+
+class Decimal(FractionalType, IKey):  # Snowflake may use Decimal as a key
     @property
     def python_type(self) -> type:
         if self.precision == 0:
@@ -66,15 +76,18 @@ class StringType(ColType):
     pass
 
 
-class IKey(ABC):
-    "Interface for ColType, for using a column as a key in data-diff"
-    python_type: type
-
-
-class ColType_UUID(StringType, IKey):
+class ColType_UUID(ColType, IKey):
     python_type = ArithUUID
 
 
+class Native_UUID(ColType_UUID):
+    pass
+
+
+class String_UUID(StringType, ColType_UUID):
+    pass
+
+
 @dataclass
 class Text(StringType):
     supported = False
