@@ -4,6 +4,7 @@ from .database_types import *
 from .base import ThreadedDatabase, import_helper, ConnectError, QueryError
 from .base import DEFAULT_DATETIME_PRECISION, DEFAULT_NUMERIC_PRECISION
 
+SESSION_TIME_ZONE = None    # Changed by the tests
 
 @import_helper("oracle")
 def import_oracle():
@@ -24,18 +25,20 @@ class Oracle(ThreadedDatabase):
     }
     ROUNDS_ON_PREC_LOSS = True
 
-    def __init__(self, host, port, user, password, *, database, thread_count, **kw):
-        assert not port
-        self.kwargs = dict(user=user, password=password, dsn="%s/%s" % (host, database), **kw)
+    def __init__(self, *, host, database, thread_count, **kw):
+        self.kwargs = dict(dsn="%s/%s" % (host, database), **kw)
 
-        self.default_schema = user
+        self.default_schema = kw.get("user")
 
         super().__init__(thread_count=thread_count)
 
     def create_connection(self):
         self._oracle = import_oracle()
         try:
-            return self._oracle.connect(**self.kwargs)
+            c = self._oracle.connect(**self.kwargs)
+            if SESSION_TIME_ZONE:
+                c.cursor().execute(f"ALTER SESSION SET TIME_ZONE = '{SESSION_TIME_ZONE}'")
+            return c
         except Exception as e:
             raise ConnectError(*e.args) from e
 
