@@ -4,9 +4,10 @@
 from abc import ABC, abstractmethod
 import time
 import os
+from numbers import Number
 from operator import attrgetter, methodcaller
 from collections import defaultdict
-from typing import List, Tuple, Iterator, Optional, Type
+from typing import List, Tuple, Iterator, Optional
 import logging
 from concurrent.futures import ThreadPoolExecutor
 
@@ -17,10 +18,11 @@ from .utils import safezip, split_space
 from .databases.base import Database
 from .databases.database_types import (
     ArithUUID,
+    IKey,
+    Native_UUID,
     NumericType,
     PrecisionType,
     StringType,
-    UnknownColType,
     Schema,
     Schema_CaseInsensitive,
     Schema_CaseSensitive,
@@ -94,11 +96,22 @@ class TableSegment:
                 "Cannot compile query when the schema is unknown. Please use TableSegment.with_schema()."
             )
 
+        col_type = self._schema[name]
         col = self._quote_column(name)
+
+        if isinstance(col_type, Native_UUID):
+            # Normalize first, apply template after (for uuids)
+            # Needed because min/max(uuid) fails in postgresql
+            col = self.database.normalize_value_by_type(col, col_type)
+            if template is not None:
+                col = template % col  # Apply template using Python's string formatting
+            return col
+
+        # Apply template before normalizing (for ints)
         if template is not None:
             col = template % col  # Apply template using Python's string formatting
 
-        return self.database.normalize_value_by_type(col, self._schema[name])
+        return self.database.normalize_value_by_type(col, col_type)
 
     def with_schema(self) -> "TableSegment":
         "Queries the table schema from the database, and returns a new instance of TableSegmentWithSchema."
@@ -211,6 +224,8 @@ class TableSegment:
                 "We recommend increasing --bisection-factor or decreasing --threads."
             )
 
+        if count:
+            assert checksum, (count, checksum)
         return count or 0, checksum if checksum is None else int(checksum)
 
     def query_key_range(self) -> Tuple[int, int]:
@@ -263,14 +278,14 @@ class TableDiffer:
 
     Parameters:
         bisection_factor (int): Into how many segments to bisect per iteration.
-        bisection_threshold (int): When should we stop bisecting and compare locally (in row count).
+        bisection_threshold (Number): When should we stop bisecting and compare locally (in row count).
         threaded (bool): Enable/disable threaded diffing. Needed to take advantage of database threads.
         max_threadpool_size (int): Maximum size of each threadpool. ``None`` means auto. Only relevant when `threaded` is ``True``.
                                    There may be many pools, so number of actual threads can be a lot higher.
     """
 
     bisection_factor: int = DEFAULT_BISECTION_FACTOR
-    bisection_threshold: int = DEFAULT_BISECTION_THRESHOLD
+    bisection_threshold: Number = DEFAULT_BISECTION_THRESHOLD  # Accepts inf for tests
     threaded: bool = True
     max_threadpool_size: Optional[int] = 1
 
@@ -305,6 +320,10 @@ class TableDiffer:
 
         key_type = table1._schema[table1.key_column]
         key_type2 = table2._schema[table2.key_column]
+        if not isinstance(key_type, IKey):
+            raise NotImplementedError(f"Cannot use column of type {key_type} as a key")
+        if not isinstance(key_type2, IKey):
+            raise NotImplementedError(f"Cannot use column of type {key_type2} as a key")
         assert key_type.python_type is key_type2.python_type
 
         # We add 1 because our ranges are exclusive of the end (like in Python)
