@@ -1,3 +1,5 @@
+import math
+
 from typing import Sequence, Optional, Tuple, Union, Dict, Any
 from uuid import UUID
 
@@ -38,3 +40,14 @@ def is_uuid(u):
     except ValueError:
         return False
     return True
+
+
+def number_to_human(n):
+    millnames = ["", "k", "m", "b"]
+    n = float(n)
+    millidx = max(
+        0,
+        min(len(millnames) - 1, int(math.floor(0 if n == 0 else math.log10(abs(n)) / 3))),
+    )
+
+    return "{:.0f}{}".format(n / 10 ** (3 * millidx), millnames[millidx])
